@@ -6,7 +6,7 @@
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
 ;; Version: 0.1
-;; Package-Requires: ((emacs "27.1") (corfu "0.38"))
+;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4") (corfu "1.0"))
 ;; Homepage: https://github.com/minad/corfu
 
 ;; This file is part of GNU Emacs.
@@ -99,10 +99,9 @@ If called with a prefix ARG, the buffer is persistent."
              (or (and (bufferp (car loc)) (car loc))
                  (find-file-noselect (car loc) t))
              (and arg (format "*corfu loc: %s*" cand)))
-          (save-restriction
-            (widen)
+          (without-restriction
             (goto-char (point-min))
-            (when-let (pos (cdr loc))
+            (when-let ((pos (cdr loc)))
               (if (bufferp (car loc))
                   (goto-char pos)
                 (forward-line (1- pos))))
