@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 0.38
-;; Package-Requires: ((emacs "27.1") (compat "29.1.4.2"))
+;; Version: 1.0
+;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4"))
 ;; Homepage: https://github.com/minad/corfu
 ;; Keywords: abbrev, convenience, matching, completion, wp
 
@@ -78,7 +78,7 @@ The value should lie between 0 and corfu-count/2."
 (defcustom corfu-continue-commands
   ;; nil is undefined command
   '(nil ignore universal-argument universal-argument-more digit-argument
-        "\\`corfu-" "\\`scroll-other-window")
+    "\\`corfu-" "\\`scroll-other-window")
   "Continue Corfu completion after executing these commands."
   :type '(repeat (choice regexp symbol)))
 
@@ -164,9 +164,9 @@ Emacs.  See also `corfu-auto-delay'."
 
 (defcustom corfu-auto-delay 0.2
   "Delay for auto completion.
-It is *not recommended* to use a very small delay or even 0,
-since this will create high load for Emacs in particular if the
-completion backend in use is expensive."
+It is *not recommended* to use a short delay or even 0, since
+this will create high load for Emacs, in particular if executing
+the completion backend is costly."
   :type 'float)
 
 (defcustom corfu-auto-commands
@@ -265,8 +265,8 @@ completion backend in use is expensive."
 (defvar-local corfu--total 0
   "Length of the candidate list `corfu--candidates'.")
 
-(defvar-local corfu--highlight #'identity
-  "Deferred candidate highlighting function.")
+(defvar-local corfu--hilit #'identity
+  "Lazy candidate highlighting function.")
 
 (defvar-local corfu--index -1
   "Index of current candidate or negative for prompt selection.")
@@ -295,7 +295,7 @@ completion backend in use is expensive."
 (defconst corfu--state-vars
   '(corfu--base
     corfu--candidates
-    corfu--highlight
+    corfu--hilit
     corfu--index
     corfu--preselect
     corfu--scroll
@@ -361,8 +361,10 @@ completion backend in use is expensive."
 
 (defun corfu--capf-wrapper (fun &optional prefix)
   "Wrapper for `completion-at-point' FUN.
-The wrapper determines if the Capf is applicable at the current position
-and performs sanity checking on the returned result.  PREFIX is a prefix
+The wrapper determines if the Capf is applicable at the current
+position and performs sanity checking on the returned result.
+For non-exclusive Capfs wrapper additionally checks if the
+current input can be completed successfully.  PREFIX is a prefix
 length override, set to t for manual completion."
   (pcase (funcall fun)
     ((and res `(,beg ,end ,table . ,plist))
@@ -420,8 +422,8 @@ length override, set to t for manual completion."
 (defun corfu--make-frame (frame x y width height buffer)
   "Show BUFFER in child frame at X/Y with WIDTH/HEIGHT.
 FRAME is the existing frame."
-  (when-let (timer (and (frame-live-p frame)
-                        (frame-parameter frame 'corfu--hide-timer)))
+  (when-let (((frame-live-p frame))
+             (timer (frame-parameter frame 'corfu--hide-timer)))
     (cancel-timer timer)
     (set-frame-parameter frame 'corfu--hide-timer nil))
   (let* ((window-min-height 1)
@@ -445,7 +447,7 @@ FRAME is the existing frame."
     ;; XXX HACK Setting the same frame-parameter/face-background is not a nop.
     ;; Check before applying the setting. Without the check, the frame flickers
     ;; on Mac. We have to apply the face background before adjusting the frame
-    ;; parameter, otherwise the border is not updated (BUG?).
+    ;; parameter, otherwise the border is not updated.
     (let* ((face (if (facep 'child-frame-border) 'child-frame-border 'internal-border))
            (new (face-attribute 'corfu-border :background nil 'default)))
       (unless (equal (face-attribute face :background frame 'default) new)
@@ -474,27 +476,10 @@ FRAME is the existing frame."
       ;; Mark window as dedicated to prevent frame reuse (#60)
       (set-window-dedicated-p win t))
     (redirect-frame-focus frame parent)
-    ;; XXX HACK: Child frame popup behavior improved on Emacs 29.
-    ;; It seems we may not need the Emacs 27/28 hacks anymore.
-    (if (eval-when-compile (< emacs-major-version 29))
-        (let (inhibit-redisplay)
-          (set-frame-size frame width height t)
-          (if (frame-visible-p frame)
-              ;; XXX HACK Avoid flicker when frame is already visible.
-              ;; Redisplay, wait for resize and then move the frame.
-              (unless (equal (frame-position frame) (cons x y))
-                (redisplay 'force)
-                (sleep-for 0.01)
-                (set-frame-position frame x y))
-            ;; XXX HACK: Force redisplay, otherwise the popup sometimes does not
-            ;; display content.
-            (set-frame-position frame x y)
-            (redisplay 'force)))
-      (set-frame-size frame width height t)
-      (unless (equal (frame-position frame) (cons x y))
-        (set-frame-position frame x y))))
-  (make-frame-visible frame)
-  frame)
+    (set-frame-size frame width height t)
+    (unless (equal (frame-position frame) (cons x y))
+      (set-frame-position frame x y)))
+  (make-frame-visible frame))
 
 (defun corfu--hide-frame-deferred (frame)
   "Deferred hiding of child FRAME."
@@ -514,50 +499,42 @@ FRAME is the existing frame."
 
 (defun corfu--move-to-front (elem list)
   "Move ELEM to front of LIST."
-  (if-let (found (member elem list))
+  (if-let ((found (member elem list)))
       (nconc (list (car found)) (delq (setcar found nil) list))
     list))
 
-;; bug#47711: Deferred highlighting for `completion-all-completions'
-;; XXX There is one complication: `completion--twq-all' already adds
-;; `completions-common-part'.
-(defun corfu--all-completions (&rest args)
-  "Compute all completions for ARGS with deferred highlighting."
-  (cl-letf* ((orig-pcm (symbol-function #'completion-pcm--hilit-commonality))
-             (orig-flex (symbol-function #'completion-flex-all-completions))
-             ((symbol-function #'completion-flex-all-completions)
-              (lambda (&rest args)
-                ;; Unfortunately for flex we have to undo the deferred
-                ;; highlighting, since flex uses the completion-score for
-                ;; sorting, which is applied during highlighting.
-                (cl-letf (((symbol-function #'completion-pcm--hilit-commonality) orig-pcm))
-                  (apply orig-flex args))))
-             ;; Defer the following highlighting functions
-             (hl #'identity)
+(defun corfu--filter-completions (&rest args)
+  "Compute all completions for ARGS with lazy highlighting."
+  (defvar completion-lazy-hilit)
+  (defvar completion-lazy-hilit-fn)
+  (cl-letf* ((completion-lazy-hilit t)
+             (completion-lazy-hilit-fn nil)
              ((symbol-function #'completion-hilit-commonality)
               (lambda (cands prefix &optional base)
-                (setq hl (lambda (x) (nconc (completion-hilit-commonality x prefix base) nil)))
-                (and cands (nconc cands base))))
-             ((symbol-function #'completion-pcm--hilit-commonality)
-              (lambda (pattern cands)
-                (setq hl (lambda (x)
-                           ;; `completion-pcm--hilit-commonality' sometimes
-                           ;; throws an internal error for example when entering
-                           ;; "/sudo:://u".
-                           (condition-case nil
-                               (completion-pcm--hilit-commonality pattern x)
-                             (t x))))
-                cands)))
-    ;; Only advise orderless after it has been loaded to avoid load order issues
-    (if (and (fboundp 'orderless-highlight-matches)
-             (fboundp 'orderless-pattern-compiler))
-        (cl-letf (((symbol-function 'orderless-highlight-matches)
-                   (lambda (pattern cands)
-                     (let ((regexps (orderless-pattern-compiler pattern)))
-                       (setq hl (lambda (x) (orderless-highlight-matches regexps x))))
-                     cands)))
-          (cons (apply #'completion-all-completions args) hl))
-      (cons (apply #'completion-all-completions args) hl))))
+                (setq completion-lazy-hilit-fn
+                      (lambda (x) (car (completion-hilit-commonality (list x) prefix base))))
+                (and cands (nconc cands base)))))
+    (if (eval-when-compile (>= emacs-major-version 30))
+        (cons (apply #'completion-all-completions args) completion-lazy-hilit-fn)
+      (cl-letf* ((orig-pcm (symbol-function #'completion-pcm--hilit-commonality))
+                 (orig-flex (symbol-function #'completion-flex-all-completions))
+                 ((symbol-function #'completion-flex-all-completions)
+                  (lambda (&rest args)
+                    ;; Unfortunately for flex we have to undo the lazy highlighting, since flex uses
+                    ;; the completion-score for sorting, which is applied during highlighting.
+                    (cl-letf (((symbol-function #'completion-pcm--hilit-commonality) orig-pcm))
+                      (apply orig-flex args))))
+                 ((symbol-function #'completion-pcm--hilit-commonality)
+                  (lambda (pattern cands)
+                    (setq completion-lazy-hilit-fn
+                          (lambda (x)
+                            ;; `completion-pcm--hilit-commonality' sometimes throws an internal error
+                            ;; for example when entering "/sudo:://u".
+                            (condition-case nil
+                                (car (completion-pcm--hilit-commonality pattern (list x)))
+                              (t x))))
+                    cands)))
+        (cons (apply #'completion-all-completions args) completion-lazy-hilit-fn)))))
 
 (defsubst corfu--length-string< (x y)
   "Sorting predicate which compares X and Y first by length then by `string<'."
@@ -607,13 +584,13 @@ FRAME is the existing frame."
                ;; bug#47678: `completion-boundaries' fails for `partial-completion'
                ;; if the cursor is moved between the slashes of "~//".
                ;; See also vertico.el which has the same issue.
-               (bounds (or (condition-case nil
-                               (completion-boundaries before table pred after)
-                             (t (cons 0 (length after))))))
+               (bounds (condition-case nil
+                           (completion-boundaries before table pred after)
+                         (t (cons 0 (length after)))))
                (field (substring str (car bounds) (+ pt (cdr bounds))))
                (completing-file (eq (corfu--metadata-get 'category) 'file))
-               (`(,all . ,hl) (corfu--all-completions str table pred pt corfu--metadata))
-               (base (or (when-let (z (last all)) (prog1 (cdr z) (setcdr z nil))) 0))
+               (`(,all . ,hl) (corfu--filter-completions str table pred pt corfu--metadata))
+               (base (or (when-let ((z (last all))) (prog1 (cdr z) (setcdr z nil))) 0))
                (corfu--base (substring str 0 base)))
     ;; Filter the ignored file extensions. We cannot use modified predicate for
     ;; this filtering, since this breaks the special casing in the
@@ -628,7 +605,7 @@ FRAME is the existing frame."
       (corfu--metadata . ,corfu--metadata)
       (corfu--candidates . ,all)
       (corfu--total . ,(length all))
-      (corfu--highlight . ,hl)
+      (corfu--hilit . ,(or hl #'identity))
       (corfu--preselect . ,(if (or (eq corfu-preselect 'prompt) (not all)
                                    (and completing-file (eq corfu-preselect 'directory)
                                         (= (length corfu--base) (length str))
@@ -731,8 +708,10 @@ FRAME is the existing frame."
   (pcase-let* ((last (min (+ corfu--scroll corfu-count) corfu--total))
                (bar (ceiling (* corfu-count corfu-count) corfu--total))
                (lo (min (- corfu-count bar 1) (floor (* corfu-count corfu--scroll) corfu--total)))
-               (`(,mf . ,acands) (corfu--affixate (funcall corfu--highlight
-                                   (seq-subseq corfu--candidates corfu--scroll last))))
+               (`(,mf . ,acands) (corfu--affixate
+                                  (cl-loop repeat corfu-count
+                                           for c in (nthcdr corfu--scroll corfu--candidates)
+                                           collect (funcall corfu--hilit (substring c)))))
                (`(,pw ,width ,fcands) (corfu--format-candidates acands))
                ;; Disable the left margin if a margin formatter is active.
                (corfu-left-margin-width (if mf 0 corfu-left-margin-width)))
@@ -746,9 +725,9 @@ FRAME is the existing frame."
 
 (defun corfu--preview-current (beg end)
   "Show current candidate as overlay given BEG and END."
-  (when-let (cand (and corfu-preview-current (>= corfu--index 0)
-                       (/= corfu--index corfu--preselect)
-                       (nth corfu--index corfu--candidates)))
+  (when-let ((cand (and corfu-preview-current (>= corfu--index 0)
+                        (/= corfu--index corfu--preselect)
+                        (nth corfu--index corfu--candidates))))
     (setq beg (+ beg (length corfu--base))
           corfu--preview-ov (make-overlay beg end nil))
     (overlay-put corfu--preview-ov 'priority 1000)
@@ -756,7 +735,12 @@ FRAME is the existing frame."
     (overlay-put corfu--preview-ov (if (= beg end) 'after-string 'display) cand)))
 
 (defun corfu--continue-p ()
-  "Continue completion?"
+  "Check if completion should be continued after a command.
+Corfu bails out if the currently selected buffer changed
+unexpectedly, if point moved to an unexpected position, if the
+input doesn't satisfy the `completion-in-region-mode--predicate'
+or if the last invoked command is not listed in
+`corfu-continue-commands'."
   (pcase-let ((pt (point))
               (buf (current-buffer))
               (`(,beg ,end . ,_) completion-in-region--data))
@@ -785,6 +769,13 @@ FRAME is the existing frame."
                             (seq-contains-p (car corfu--input) corfu-separator)))
                    (funcall completion-in-region-mode--predicate)))))))
 
+(defun corfu--window-change (_)
+  "Window and buffer change hook which quits Corfu."
+  (let ((buf (current-buffer))
+        (beg (car completion-in-region--data)))
+    (unless (and beg (eq buf (marker-buffer beg)) (eq buf (window-buffer)))
+      (corfu-quit))))
+
 (defun corfu--post-command ()
   "Refresh Corfu after last command."
   (if (corfu--continue-p)
@@ -813,8 +804,8 @@ FRAME is the existing frame."
   (activate-change-group (setq corfu--change-group (prepare-change-group)))
   (setcdr (assq #'completion-in-region-mode minor-mode-overriding-map-alist) corfu-map)
   (add-hook 'pre-command-hook #'corfu--prepare nil 'local)
-  (add-hook 'window-selection-change-functions #'corfu-quit nil 'local)
-  (add-hook 'window-buffer-change-functions #'corfu-quit nil 'local)
+  (add-hook 'window-selection-change-functions #'corfu--window-change nil 'local)
+  (add-hook 'window-buffer-change-functions #'corfu--window-change nil 'local)
   (add-hook 'post-command-hook #'corfu--post-command)
   ;; Disable default post-command handling, since we have our own
   ;; checks in `corfu--post-command'.
@@ -919,7 +910,7 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
       (`(,fun ,beg ,end ,table . ,plist)
        (let ((completion-in-region-mode-predicate
               (lambda ()
-                (when-let (newbeg (car-safe (funcall fun)))
+                (when-let ((newbeg (car-safe (funcall fun))))
                   (= newbeg beg))))
              (completion-extra-properties plist))
          (setq completion-in-region--data
@@ -942,8 +933,8 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
              (corfu--popup-support-p))
     (if (<= corfu-auto-delay 0)
         (corfu--auto-complete-deferred)
-      ;; NOTE: Do not use idle timer since this leads to unacceptable slowdowns,
-      ;; in particular if flyspell-mode is enabled.
+      ;; Do not use idle timer since this leads to unpredictable pauses, in
+      ;; particular with `flyspell-mode'.
       (setq corfu--auto-timer
             (run-at-time corfu-auto-delay nil
                          #'corfu--auto-complete-deferred (corfu--auto-tick))))))
@@ -1030,11 +1021,11 @@ A scroll bar is displayed from LO to LO+BAR."
 (cl-defgeneric corfu--affixate (cands)
   "Annotate CANDS with annotation function."
   (setq cands
-        (if-let (aff (or (corfu--metadata-get 'affixation-function)
-                         (plist-get corfu--extra :affixation-function)))
+        (if-let ((aff (or (corfu--metadata-get 'affixation-function)
+                          (plist-get corfu--extra :affixation-function))))
             (funcall aff cands)
-          (if-let (ann (or (corfu--metadata-get 'annotation-function)
-                           (plist-get corfu--extra :annotation-function)))
+          (if-let ((ann (or (corfu--metadata-get 'annotation-function)
+                            (plist-get corfu--extra :annotation-function))))
               (cl-loop for cand in cands collect
                        (let ((suffix (or (funcall ann cand) "")))
                          ;; The default completion UI adds the
@@ -1064,6 +1055,10 @@ A scroll bar is displayed from LO to LO+BAR."
   ;; Ensure that state is initialized before next Corfu command
   (when (and (symbolp this-command) (string-prefix-p "corfu-" (symbol-name this-command)))
     (corfu--update))
+  ;; If the next command is not listed in `corfu-continue-commands', insert the
+  ;; currently selected candidate and bail out of completion. This way you can
+  ;; continue typing after selecting a candidate. The candidate will be inserted
+  ;; and your new input will be appended.
   (when (and (eq corfu-preview-current 'insert)
              (/= corfu--index corfu--preselect)
              ;; See the comment about `overriding-local-map' in `corfu--post-command'.
@@ -1103,8 +1098,8 @@ AUTO is non-nil when initializing auto completion."
 (cl-defgeneric corfu--teardown ()
   "Tear-down Corfu."
   (corfu--popup-hide)
-  (remove-hook 'window-selection-change-functions #'corfu-quit 'local)
-  (remove-hook 'window-buffer-change-functions #'corfu-quit 'local)
+  (remove-hook 'window-selection-change-functions #'corfu--window-change 'local)
+  (remove-hook 'window-buffer-change-functions #'corfu--window-change 'local)
   (remove-hook 'pre-command-hook #'corfu--prepare 'local)
   (remove-hook 'post-command-hook #'corfu--post-command)
   (when corfu--preview-ov (delete-overlay corfu--preview-ov))
@@ -1115,7 +1110,7 @@ AUTO is non-nil when initializing auto completion."
   "Sort LIST by length and alphabetically."
   (sort list #'corfu--length-string<))
 
-(defun corfu-quit (&rest _)
+(defun corfu-quit ()
   "Quit Corfu completion."
   (interactive)
   (completion-in-region-mode -1))
@@ -1131,9 +1126,13 @@ there hasn't been any input, then quit."
         (corfu--goto -1)
         (setq this-command #'corfu-first))
     ;; Cancel all changes and start new change group.
-    (cancel-change-group corfu--change-group)
-    (activate-change-group (setq corfu--change-group (prepare-change-group)))
-    (when (eq last-command #'corfu-reset) (corfu-quit))))
+    (pcase-let* ((`(,beg ,end . ,_) completion-in-region--data)
+                 (str (buffer-substring-no-properties beg end)))
+      (cancel-change-group corfu--change-group)
+      (activate-change-group (setq corfu--change-group (prepare-change-group)))
+      ;; Quit when resetting, when input did not change.
+      (when (equal str (buffer-substring-no-properties beg end))
+        (corfu-quit)))))
 
 (defun corfu-insert-separator ()
   "Insert a separator character, inhibiting quit on completion boundary.
@@ -1270,8 +1269,11 @@ Quit if no candidate is selected."
 (defcustom global-corfu-modes t
   "List of modes where Corfu should be enabled.
 The variable can either be t, nil or a list of t, nil, mode
-symbols or elements of the form (not modes)."
-  :type '(repeat sexp))
+symbols or elements of the form (not modes).  Examples:
+  - Enable everywhere, except in Org: ((not org-mode) t).
+  - Enable in programming modes except Python: ((not python-mode) prog-mode).
+  - Enable only in text modes: (text-mode)."
+  :type '(choice (const t) (repeat sexp)))
 
 ;;;###autoload
 (define-globalized-minor-mode global-corfu-mode
@@ -1288,7 +1290,7 @@ symbols or elements of the form (not modes)."
                                   ('t t)
                                   ('nil 0)
                                   ((pred symbolp) (and (derived-mode-p p) t))
-                                  (`(not . ,m) (and (apply #'derived-mode-p m) 0)))))))
+                                  (`(not . ,m) (and (seq-some #'derived-mode-p m) 0)))))))
     (corfu-mode 1)))
 
 ;; Emacs 28: Do not show Corfu commands with M-X
